#include <fstream>
#include <stdlib.h>
#include <stdio.h>
#include "cdbg.h"
#include "mp3bg.h"
#include "control.h"
#include "logfile.h"

using namespace GameLauncher;
using LogFile::gl_log;

Background_Music_Controller::~Background_Music_Controller()
{
}

void
Null_Music_Controller::load_config(Config_File * config_file)
{
}

void
Null_Music_Controller::save_state(Config_File * state_file)
{
}

void
Null_Music_Controller::restore_state(Config_File * state_file)
{
}

void
Null_Music_Controller::skip_to_next_song()
{
}

void
Null_Music_Controller::start_song_over()
{
}

#if USE_CD

#define _S(_A_, _B_) \
char * CD_Music_Controller:: _A_ = _B_

_S( CONFIG_SECTION,             "Main");
_S( CONFIG_CD_VOLUME,           "cd_volume");

_S( STATE_SECTION,              "Main");
_S( STATE_LAST_CD_FRAME,        "last_cd_frame");
_S( STATE_LAST_CD_TOTAL_FRAMES, "last_cd_total_frames");

#undef _S


CD_Music_Controller::CD_Music_Controller(Engine * engine)
{
    p_cd_background     = CD_Background::create();

    engine->set_background_task(p_cd_background);
}

CD_Music_Controller::~CD_Music_Controller()
{
    delete p_cd_background;
}

void
CD_Music_Controller::load_config(Config_File * config_file)
{
    int volume;
    Config_Section *    section;

    section     = config_file->get_section(CONFIG_SECTION);
    volume      = section->get_int(CONFIG_CD_VOLUME, 0);
    delete section;

    if (volume > 1000)
        volume = 1000;
    if (volume < 0)
        volume = 0;

    p_cd_background->set_volume(volume);
}

void
CD_Music_Controller::restore_state(Config_File * state_file)
{
    int last_frame;
    int last_total_frames;

    Config_Section *    section;

    section             = state_file->get_section(STATE_SECTION);
    last_frame          = section->get_int(STATE_LAST_CD_FRAME, 0);
    last_total_frames   = section->get_int(STATE_LAST_CD_TOTAL_FRAMES, 0);
    delete section;

    // Only use last frame if the CD is the same as the last.
    // Otherwise start over from the beginning of the CD.
    int current_total_frames;
    current_total_frames        = p_cd_background->total_frames();
    if (current_total_frames != last_total_frames)
        last_frame = 0;

    p_cd_background->seek(last_frame);
}

void
CD_Music_Controller::save_state(Config_File * state_file)
{
    int current_frame;
    int total_frames;

    current_frame       = p_cd_background->current_frame();
    total_frames        = p_cd_background->total_frames();

    Config_Section *    section;
    section     = state_file->get_section(STATE_SECTION);
    section->set_int(STATE_LAST_CD_FRAME, current_frame);
    section->set_int(STATE_LAST_CD_TOTAL_FRAMES, total_frames);
    delete section;
}

void
CD_Music_Controller::skip_to_next_song()
{
}

void
CD_Music_Controller::start_song_over()
{
}

#endif // USE_CD == 1



#if (USE_MP3 == 1)

#define _S(_A_, _B_) \
char * MP3_Music_Controller:: _A_ = _B_

_S( CONFIG_SECTION,             "Main");
_S( CONFIG_MP3_PLAYLIST_FILE,   "mp3_playlist_file");
_S( CONFIG_MP3_VOLUME,          "mp3_volume");

_S( STATE_SECTION,              "Main");
_S( STATE_LAST_MP3_FILE,        "last_mp3_file");
_S( STATE_LAST_MP3_POSITION,    "last_mp3_position");
_S( STATE_LAST_MP3_FILE_INDEX,  "last_mp3_index");

#undef _S


MP3_Music_Controller::MP3_Music_Controller(Engine * engine)
{
    p_mp3_background    = MP3_Background::create();
    engine->set_background_task(p_mp3_background);
}

MP3_Music_Controller::~MP3_Music_Controller()
{
    delete p_mp3_background;
}

void
MP3_Music_Controller::load_config(Config_File * config_file)
{
    Config_Section *    section;
    string      playlist_file;
    int         volume;

    section         = config_file->get_section(CONFIG_SECTION);
    playlist_file   = section->get_string(CONFIG_MP3_PLAYLIST_FILE, "");
    volume          = section->get_int(CONFIG_MP3_VOLUME, 1000);
    delete section;

    if (volume > 1000)
        volume = 1000;
    if (volume < 0)
        volume = 0;

    read_playlist(playlist_file, p_playlist);
    p_mp3_background->set_playlist(p_playlist);
    p_mp3_background->set_volume(volume);
}

void
MP3_Music_Controller::restore_state(Config_File * state_file)
{
    string      last_mp3_file;
    int         last_position;
    unsigned    last_playlist_index;

    Config_Section *    section;

    section             = state_file->get_section(STATE_SECTION);
    last_mp3_file       = section->get_string(STATE_LAST_MP3_FILE, "");
    last_position       = section->get_int(STATE_LAST_MP3_POSITION, 0);
    last_playlist_index =
        (unsigned) section->get_int(STATE_LAST_MP3_FILE_INDEX, 0);
    delete section;

    // Only set position and index if the last file matches the index.
    if ((last_playlist_index < p_playlist.size()) &&
        (last_mp3_file == p_playlist[last_playlist_index]))
    {
        p_mp3_background->seek(last_playlist_index, last_position);
    }
}


void
MP3_Music_Controller::save_state(Config_File * state_file)
{
    string      last_mp3_file;
    int         last_position;
    unsigned    last_playlist_index;

    last_mp3_file       = p_mp3_background->get_current_mp3_name();
    last_position       = p_mp3_background->get_current_position();
    last_playlist_index = p_mp3_background->get_current_mp3_index();

    Config_Section *    section;

    section     = state_file->get_section(STATE_SECTION);
    section->set_string(STATE_LAST_MP3_FILE, last_mp3_file);
    section->set_int(STATE_LAST_MP3_POSITION, last_position);
    section->set_int(STATE_LAST_MP3_FILE_INDEX, last_playlist_index);
    delete section;
}

void
MP3_Music_Controller::read_playlist(const string & playlist_file,
                                         vector<string> & playlist)
{
    playlist.resize(0);

    if (playlist_file == "")
        return;

    ifstream    playlist_stream(playlist_file.c_str());

    while(! playlist_stream.eof())
    {
        string  line;
        string  mp3_file;
        string::size_type start, end, len;

        getline(playlist_stream, line);

        start = 0;
        while ((start < line.size()) && (isspace(line[start])))
            start++;
        if (start == line.size())
            continue;

        // Skip comments
        if (line[start] == '#')
            continue;

        // end will point to character before next space
        end = start+1;
        while ((end < line.size()) && (! isspace(line[end])))
            end++;

        end--;
        len = end - start + 1;

        mp3_file = line.substr(start, len);
        playlist.push_back(mp3_file);
    }
    
    char buf[80];
    sprintf(buf, "MP3 playlist contains %lu files",
            (unsigned long) playlist.size());
    gl_log->info(buf);
}

void
MP3_Music_Controller::skip_to_next_song()
{
    p_mp3_background->skip_to_next_song();
}

void
MP3_Music_Controller::start_song_over()
{
    p_mp3_background->start_song_over();
}

#endif // USE_MP3 == 1
