#include "enginei.h"
#include "logfile.h"

using namespace GameEngine;
using namespace std;
using LogFile::gl_log;

Timer_Manager::Timer_Manager()
    : p_timers()
{
    p_ticks_per_second = 1;

    return;
}

Timer *
Timer_Manager::create_timer()
{
    Timer_Impl *        timer;

    timer = new Timer_Impl(this, p_ticks_per_second);
    if (timer == 0)
        return timer;

    p_timers.push_back(timer);

    return timer;
}


void
Timer_Manager::delete_timer(Timer * timer)
{
    Timer_List_Impl::iterator   i;

    for (i = p_timers.begin(); i != p_timers.end(); i++)
    {
        if (*i == timer)
        {
            p_timers.erase(i);
            break;
        }
    }

    return;
}

bool
Timer_Manager::update()
{
    Timer_List_Impl::iterator   i;

    for (i = p_timers.begin(); i != p_timers.end(); i++)
    {
        (*i)->tick_tock();
    }

    // Timers don't directly cause any visual updates
    return false;
}

void
Timer_Manager::adjust_frequency(unsigned int ticks_per_second)
{
    Timer_List_Impl::iterator   i;

    p_ticks_per_second = ticks_per_second;

    for (i = p_timers.begin(); i != p_timers.end(); i++)
    {
        (*i)->adjust_frequency(p_ticks_per_second);
    }
}

Timer_Impl::Timer_Impl(Timer_Manager * timer_manager, int ticks_per_second)
{
    p_timer_manager     = timer_manager;
    p_ticks_per_second  = ticks_per_second;
    p_interval          = 0;
    p_ticks_left        = 0;
    p_state             = STOPPED;
    return;
}

Timer_Impl::~Timer_Impl()
{
    p_timer_manager->delete_timer(this);
    return;
}

void
Timer_Impl::set(unsigned int milliseconds)
{
    p_ticks_left        = (milliseconds * p_ticks_per_second) / 1000;
    p_interval          = 0;
    p_state             = RUNNING;
    return;
}

void
Timer_Impl::set_interval(unsigned int milliseconds)
{
    p_ticks_left        = (milliseconds * p_ticks_per_second) / 1000;
    p_interval          = p_ticks_left;
    p_state             = RUNNING;
    return;
}

void
Timer_Impl::disable()
{
    p_ticks_left        = 0;
    p_interval          = 0;
    p_state             = STOPPED;
    return;
}

bool
Timer_Impl::expired()
{
    return ((p_state == RUNNING) && (p_ticks_left == 0));
}

bool
Timer_Impl::stopped()
{
    return (p_state == STOPPED);
}

void
Timer_Impl::tick_tock()
{
    if (p_state == STOPPED)
        return;

    if ((p_ticks_left == 0) && (p_interval == 0))
    {
        p_state = STOPPED;
    }
    else if ((p_ticks_left == 0) && (p_interval != 0))
    {
        p_ticks_left = p_interval;
    }

    p_ticks_left--;
    return;
}

void
Timer_Impl::adjust_frequency(unsigned int ticks_per_second)
{
    fix ticks_left_ms;
    fix interval_ms;

    if (p_state == STOPPED)
        return;

    // Convert back to milliseconds
    ticks_left_ms       = (p_ticks_left * 1000) / p_ticks_per_second;
    interval_ms         = (p_interval * 1000) / p_ticks_per_second;

    // Update frequency and recalculate ticks
    p_ticks_per_second  = ticks_per_second;
    p_ticks_left        = (ticks_left_ms * fix(p_ticks_per_second)) / 1000;
    p_interval          = (interval_ms * fix(p_ticks_per_second)) / 1000;
}

