#include "gui/separate.h"
#include "gui/uimgr.h"

using namespace GUI;

/************************************************************************
 *
 * Separator
 *
 ************************************************************************/

Separator::Separator(int x, int y, int width, int height)
{
    p_bounding_box.set_bounds(x, y, width, height);
    p_justification = CENTER_JUSTIFIED;
    p_changed = true;
    p_ui = UI_Manager::get_separator_ui();
    p_ui->install_separator(this);
}

Separator::Separator(const Rectangle & bounding_box)
{
    p_bounding_box = bounding_box;
    p_justification = CENTER_JUSTIFIED;
    p_changed = true;
    p_ui = UI_Manager::get_separator_ui();
    p_ui->install_separator(this);
}

Separator::~Separator()
{
    delete p_ui;
}

const Rectangle *
Separator::get_bounding_box() const
{
    return &p_bounding_box;
}

void
Separator::set_justification(Justification justification)
{
    p_justification = justification;
    p_changed = true;
}

Justification
Separator::get_justification() const
{
    return p_justification;
}

bool
Separator::update()
{
    if (p_changed)
    {
        p_ui->separator_changed();
        p_changed = false;
        return true;
    }

    return false;
}

void
Separator::draw(BITMAP * frame)
{
    p_ui->draw(frame);
}

/************************************************************************
 *
 * Separator_UI
 *
 ************************************************************************/

Separator_UI::~Separator_UI()
{
}


/************************************************************************
 *
 * Abstract_Separator_UI
 *
 ************************************************************************/

Abstract_Separator_UI::Abstract_Separator_UI()
{
    p_black = makecol(0, 0, 0);
    p_white = makecol(255, 255, 255);
}


void
Abstract_Separator_UI::install_separator(Separator * separator)
{
    p_separator = separator;
}

void
Abstract_Separator_UI::separator_changed()
{
    p_bounding_box = p_separator->get_bounding_box();

    p_inner_box.set_bounds(get_inner_x(), get_inner_y(),
                           p_bounding_box->get_width() / 2,
                           p_bounding_box->get_height() / 2);

    p_outer_box.set_bounds(p_inner_box.get_x() - OUTLINE_SIZE,
                           p_inner_box.get_y() - OUTLINE_SIZE,
                           p_inner_box.get_width() + OUTLINE_SIZE * 2,
                           p_inner_box.get_height() + OUTLINE_SIZE * 2);
}

void
Abstract_Separator_UI::draw(BITMAP * frame)
{
    rectfill(frame, p_outer_box.get_x(), p_outer_box.get_y(),
             p_outer_box.get_lower_right_x(), p_outer_box.get_lower_right_y(),
             p_black);

    rectfill(frame, p_inner_box.get_x(), p_inner_box.get_y(),
             p_inner_box.get_lower_right_x(), p_inner_box.get_lower_right_y(),
             p_white);
}
