#include "allegro.h"
#include "engine.h"
#include "gui.h"
#include "gui/horiz.h"
#include "gui/rot270.h"
#include "gui/rot90.h"
#include "logfile.h"
#include "allegttf.h"
#include <stdio.h>

using namespace GUI;
using namespace GameEngine;
using LogFile::gl_log;


static RGB_MAP s_rgb_table;
static Engine * s_engine = 0;

class Test_List_Model : public List_Model
{
  public:
    Test_List_Model(unsigned size);
    virtual string get_item_text_at(unsigned index) const;
    virtual Icon * get_item_icon_at(unsigned index) const;
    virtual unsigned get_size() const;

  private:
    unsigned p_size;
};

Test_List_Model::Test_List_Model(unsigned size)
{
    p_size = size;
}

string
Test_List_Model::get_item_text_at(unsigned index) const
{
    char text[100];
    sprintf(text, "Test %0*u", (index % 15) + 1, index);
    return string(text);
}

Icon *
Test_List_Model::get_item_icon_at(unsigned index) const
{
    return 0;
}

unsigned
Test_List_Model::get_size() const
{
    return p_size;
}

class Test_Class : public Updatable, public Drawable
{
  public:
    Test_Class(string justification);
    ~Test_Class();
    bool update();
    void draw(BITMAP * frame);

  private:
    Menu * p_test;
};

Test_Class::Test_Class(string justification)
{
    Test_List_Model * model;

    model = new Test_List_Model(100);
    p_test = new Menu(SCREEN_W/4, SCREEN_H/4, SCREEN_W/2, SCREEN_H/2);
    p_test->set_title("Title");
    p_test->set_subtitle("Sub-Title");
    p_test->set_list_model(model);
    p_test->set_selection_position(25, 28);
    p_test->set_unselected_colors(makecol(255, 255, 255), makecol(0, 0, 0));
    p_test->set_selected_colors(makecol(255, 0, 0), makecol(192, 192, 64));

    if (justification == "l")
        p_test->set_justification(LEFT_JUSTIFIED);
    else if (justification == "r")
        p_test->set_justification(RIGHT_JUSTIFIED);
    else
        p_test->set_justification(CENTER_JUSTIFIED);
}

Test_Class::~Test_Class()
{
    delete p_test;
}

bool
Test_Class::update()
{
    while (keypressed())
    {
        int key = (readkey() >> 8);

        if (key == KEY_ESC)
        {
            s_engine->stop();
        }

        if (key == KEY_5)
            p_test->unhide();

        if (key == KEY_6)
            p_test->hide_split();

        if (key == KEY_7)
            p_test->hide_left();

        if (key == KEY_8)
            p_test->hide_right();

        if (key == KEY_9)
            p_test->hide_split_center_selected(1000);

        if (key == KEY_UP)
        {
            if ((key_shifts & KB_SHIFT_FLAG) == 0)
                p_test->line_up(SMOOTH_SCROLL);
            else
                p_test->line_up(INSTANT_SCROLL);
        }

        if (key == KEY_LEFT)
        {
            if ((key_shifts & KB_SHIFT_FLAG) == 0)
                p_test->page_up(SMOOTH_SCROLL);
            else
                p_test->page_up(INSTANT_SCROLL);
        }

        if (key == KEY_DOWN)
        {
            if ((key_shifts & KB_SHIFT_FLAG) == 0)
                p_test->line_down(SMOOTH_SCROLL);
            else
                p_test->line_down(INSTANT_SCROLL);
        }

        if (key == KEY_RIGHT)
        {
            if ((key_shifts & KB_SHIFT_FLAG) == 0)
                p_test->page_down(SMOOTH_SCROLL);
            else
                p_test->page_down(INSTANT_SCROLL);
        }

    }
    
    return p_test->update();
}

void
Test_Class::draw(BITMAP * frame)
{
#if 1
    clear(frame);
#else
    clear_to_color(frame, makecol(0, 0, 128));
#endif
    p_test->draw(frame);
}

int main(int argc, char * argv[])
{
    FONT * ttf_font;
    UI_Factory * factory;
    Test_Class * test;
    string fullscreen_string;
    string rotation_string;
    string justification_string;
    bool fullscreen;

    if (argc != 4)
    {
        cerr << "Usage: test <fullscreen> <rotation> <justification>\n";
        return(1);
    }

    LogFile::open_log_file("logs/test.log");

    allegro_init();
    install_keyboard();
    
    fullscreen_string = argv[1];
    rotation_string = argv[2];
    justification_string = argv[3];

    if (fullscreen_string == "y")
        fullscreen = true;
    else
        fullscreen = false;

    s_engine = Engine::get_instance();
    s_engine->set_graphics_mode(800, 600, 16, fullscreen);
    s_engine->set_animation_mode(Engine::PAGE_FLIPPING, true, true);

    {
        PALETTE palette;
        RGB     black = { 0, 0, 0 };

        generate_332_palette(palette);
        palette[0] = black;

        set_palette(palette);

        create_rgb_table(&s_rgb_table, palette, NULL);
        rgb_map = &s_rgb_table;
        antialias_init(palette);
    }

    ttf_font = load_ttf_font("../slicker.ttf", 18, ALLEGTTF_REALSMOOTH);

    if (rotation_string == "l")
        factory = new Rotate_270_UI_Factory();
    else if (rotation_string == "r")
        factory = new Rotate_90_UI_Factory();
    else
        factory = new Horizontal_UI_Factory();

    UI_Manager::set_engine(s_engine);
    UI_Manager::set_font(ttf_font);
    UI_Manager::install_ui_factory(factory);

#if 0
    bitmap = load_bitmap("../datafile/camera.tga", 0);
    icon = new Bitmap_Icon(bitmap);
    label = new Label(200, 300);
    label->set_outline_color(makecol(192, 192, 64), 800);
    label->set_background_color(makecol(255, 0, 255));
    label->set_contents("Test", icon);
#endif

    test = new Test_Class(justification_string);
    s_engine->get_root_container()->add_drawable(test, 0);
    s_engine->get_root_container()->add_updatable(test, 0);
    s_engine->run();

    delete test;
    delete factory;
    delete s_engine;

    return 0;
}
END_OF_MAIN();
