#ifndef _GUI_ROT90_H_
#define _GUI_ROT90_H_

#include "gui.h"

namespace GUI
{
    /**
     * A factory for all UI components rotated 90 degrees clockwise.
     */
    class Rotate_90_UI_Factory : public UI_Factory
    {
      public:
        virtual Label_UI * create_label_ui();

        virtual List_UI * create_list_ui();

        virtual Menu_UI * create_menu_ui();

        virtual Separator_UI * create_separator_ui();
    };
    
    /**
     * An implementation of Label_UI where the text is rotated 90
     * degress clockwise.
     */
    class Rotate_90_Label_UI : public Abstract_Label_UI
    {
      public:
        Rotate_90_Label_UI();

        virtual void install_ui(Label * label);

        virtual void label_updated();

      private:
        fixed p_90_degrees;
    };

    /**
     * An implementation of List_UI where the text is rotated 90
     * degress clockwise.
     */
    class Rotate_90_List_UI : public Abstract_List_UI
    {
      public:
        virtual void install_list(List * list);
        
      protected:
        virtual void hide_item_left(Label * item, int index, int milliseconds);

        virtual void hide_item_right(Label * item, int index,
                                     int milliseconds);
        
        virtual void hide_item_top(Label * item, int index,
                                   int milliseconds);

        virtual void center_item(Label * item, int milliseconds);
        
        virtual void unhide_item(Label * item, int index, int milliseconds);

        virtual void hide_offscreen_item(Label * item);

      private:
        /**
         * Gets the offset from the left edge of the list with the
         * current justification.  A label is passed in since the
         * width of the label can affect the offset.
         *
         * @param label Label to use when calculating offset.
         * @return The offset from the left edge of the list.
         */
        int get_justification_offset(Label * label);

        /**
         * Gets the x-coordinate of the item relative from the right
         * edge of the list.
         *
         * @param index Index relative to the item on the right edge
         * of the screen.
         * @return X-coordinate for the item at this index.
         */
        int get_indexed_x(int index);
    };

    /**
     * An implementation of Menu_UI where the text is rotated 90
     * degress clockwise.
     */
    class Rotate_90_Menu_UI : public Abstract_Menu_UI
    {
      public:
        virtual void install_menu(Menu * menu);
    };

    /**
     * An implementation of Separator_UI where the text is rotated 90
     * degress clockwise.
     */
    class Rotate_90_Separator_UI : public Abstract_Separator_UI
    {
      protected:
        virtual int get_inner_x();

        virtual int get_inner_y();
    };
};

#endif

/* Local Variables: */
/* mode: c++ */
/* End: */
