#ifndef _OUTLINE_H_
#define _OUTLINE_H_

#include "allegro.h"
#include <string>

/**
 * Functions for writing text with an outline.
 */
namespace Outline
{
    /**
     * Calculates the height of the font taking into account the extra
     * padding of the outline.
     *
     * @param font Font to use.
     * @return Height of font.
     */
    int outline_text_height(const FONT * font);

    /**
     * Calculates the length (in pixels) of string taking into account
     * the extra padding for the outline.
     *
     * @param font Font to use.
     * @param text String to calculate length of.
     * @return Length of string.
     */
    int outline_text_length(const FONT * font, const string & text);

    /**
     * Writes text with an outline to a specified bitmap.
     *
     * @param bitmap Bitmap to write on.
     * @param font Font to use for text.
     * @param text Text to write.
     * @param foreground_color Color of the text.
     * @param outline_color Color of the outline.
     */
    void outline_textout(BITMAP * bitmap, const FONT * font,
                         const string & text, int foreground_color,
                         int outline_color);
};



#endif

/* Local Variables: */
/* mode: c++ */
/* End: */
