#ifndef _SSHOT_H_
#define _SSHOT_H_

#include "allegro.h"
#include "engine.h"

namespace GameLauncher
{

    using namespace GameEngine;

    class Screen_Shot_Listener;

    class Screen_Shot : public Drawable, public Updatable
    {
      public:
        Screen_Shot(Engine * engine);
        virtual ~Screen_Shot();

        virtual void    draw(BITMAP * frame);
        virtual bool    update();

        void            set_bitmap(const string & bitmap_file,
                                   fix brightness_percentage, int delay = 0);
        void            cancel_set_bitmap();
        void            set_listener(Screen_Shot_Listener * listener);
        void            set_toggle_key(int scancode);
        void            set_prefer_integer_scaling(bool
                                                   prefer_integer_scaling);

      private:
        enum State {SHOWING,
                    WAITING};
                    
        void            update_bitmap();

        BITMAP *        p_unmodified_screen_shot;
        BITMAP *        p_screen_shot;
        bool            p_use_bitmap;
        bool            p_requires_loading;
        State           p_state;
        string          p_bitmap_file;
        string          p_new_bitmap_file;
        fix             p_brightness_percentage;
        Timer *         p_delay_timer;
        bool            p_prefer_integer_scaling;
        int             p_width;
        int             p_height;
        int             p_x;
        int             p_y;
        
        Keypress_Watcher *      p_toggle_key;
        Screen_Shot_Listener *  p_listener;
        Screen_Shot_Listener *  p_null_listener;
    };

    class Screen_Shot_Listener
    {
      public:
        virtual void    toggle_screen_shot() = 0;
    };

    class Null_Screen_Shot_Listener : public Screen_Shot_Listener
    {
      public:
        void            toggle_screen_shot();
    };
};


#endif

/* Local Variables: */
/* mode: c++ */
/* End: */
