#include <windows.h>
#include <stdio.h>
#include <string>
#include "exit.h"

HINSTANCE hInst;
HWND hwndMain;

extern "C"
{
  LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
  int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR lpCmdLine, int nCmdShow);
}

static bool createMainWindow();
static bool execCommand(string command, DWORD * exitCode);
static const char * getVersion();
static void removeCursor();
static void displayCursor();
static void blankScreen();
static void unblankScreen();

// This code is based on code from Mike Billings' Cocktail Frontend
// http://mikebillings.com/frontend/

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
    // Set global reference to instance
    hInst = hInstance;

    if (!createMainWindow())
    {
        return FALSE;
    }
    blankScreen();

    string runIt = getenv("COMSPEC") + string(" /c runit.bat");
    DWORD exitCode;
    bool execSuccess = execCommand("_gl32.exe 0x0bad 0xcafe", &exitCode);
    if (! execSuccess)
    {
        unblankScreen();
        MessageBox(hwndMain, "Failed to run _gl32.exe", "Error", MB_OK);
    }
    else
    {
        while ((exitCode == EXIT_RUN_BATCH_FILE) ||
               (exitCode == EXIT_RUN_DEBUGGED_BATCH_FILE))
        {
            if (exitCode == EXIT_RUN_DEBUGGED_BATCH_FILE)
            {
                unblankScreen();
                (void) execCommand(runIt, &exitCode);
                blankScreen();
            }
            else
            {
                (void) execCommand(runIt, &exitCode);
            }
            execSuccess = execCommand("_gl32.exe 0xdead 0xbeef", &exitCode);
            if (! execSuccess)
            {
                unblankScreen();
                MessageBox(hwndMain, "Failed to run _gl32.exe", "Error",
                           MB_OK);
            }
        }
    }

    unblankScreen();
    if (exitCode == EXIT_ERROR)
    {
        MessageBox(hwndMain,
                   "An error occured. "
                   "See \"logs/glaunch.log\" for more information",
                   "Error", MB_OK);
    }

    if (exitCode == EXIT_NO_LOG_DIRECTORY)
    {
        MessageBox(hwndMain,
                   "Could not create \"logs\" directory",
                   "Error", MB_OK);
    }

    return 0;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg)
    {
        case WM_CREATE:
            break;
        case WM_SIZE:
            break;
        case WM_PAINT:
            break;
        case WM_DESTROY:
            PostQuitMessage(0);
            break;
        default:
            return(DefWindowProc(hwnd, uMsg, wParam, lParam));
    }
    return 0;
}

static bool createMainWindow()
{
    WNDCLASS wc;
    wc.style = 0;
    wc.lpfnWndProc = (WNDPROC) WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInst;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);
    wc.lpszMenuName =  "MainMenu";
    wc.lpszClassName = "MainWndClass";
    if (!RegisterClass(&wc))
    {
        MessageBox(NULL, "Failed to register window class!", "Error", MB_OK);
        return false;
    }

    hwndMain = CreateWindow("MainWndClass", getVersion(), WS_POPUP,
                            CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                            CW_USEDEFAULT, NULL, NULL, hInst, NULL);
    if (!hwndMain)
    {
        MessageBox(NULL, "Failed to create window!", "Error", MB_OK);
        return false;
    }

    ShowWindow(hwndMain, SW_SHOWDEFAULT);
    return true;
}

static const char * getVersion()
{
    return "Game Launcher v" GLAUNCH_VERSION;
}

static bool execCommand(string command, DWORD * exitCode)
{
    SECURITY_ATTRIBUTES sa;
    sa.nLength = sizeof(SECURITY_ATTRIBUTES);
    sa.lpSecurityDescriptor = NULL;
    sa.bInheritHandle = TRUE;
	
    STARTUPINFO si;
    memset(&si, 0, sizeof(STARTUPINFO));
    si.cb = sizeof(STARTUPINFO);
    si.dwFlags = STARTF_USESHOWWINDOW;
    si.wShowWindow = SW_SHOW;
	
    DWORD creationFlags = 0;
    PROCESS_INFORMATION proc;
    if (CreateProcess(NULL, (char *) command.c_str(), &sa, NULL, TRUE,
                      creationFlags, NULL, NULL, &si, &proc))
    {
        // Wait for the shelled application to finish
        WaitForSingleObject(proc.hProcess, INFINITE);
        if (exitCode != 0)
        {
            if (!GetExitCodeProcess(proc.hProcess, exitCode))
            {
                MessageBox(hwndMain, "GetExitCodeProcess failed!", "Error",
                           MB_OK);
            }
        }
        CloseHandle(proc.hProcess);
        CloseHandle(proc.hThread);
        return true;
    }
    else
    {
        DWORD last_error;
        char message[255];
	
        last_error = GetLastError();
        sprintf(message, "Failed to execute [%s] %ld", command.c_str(),
                last_error);
        MessageBox(hwndMain, message, "Error", MB_OK);
        return false;
    }
}

void removeCursor()
{
    while (ShowCursor(FALSE) >= 0)
        /* empty */ ;
}

void displayCursor()
{
    while (ShowCursor(TRUE) < 0)
        /* empty */ ;
}

static void blankScreen()
{
    removeCursor();
    int cx = GetSystemMetrics(SM_CXSCREEN);
    int cy = GetSystemMetrics(SM_CYSCREEN);
    SetWindowPos(hwndMain, HWND_TOPMOST, 0, 0, cx, cy, SWP_SHOWWINDOW);
    UpdateWindow(hwndMain);
}

static void unblankScreen()
{
    displayCursor();
    ShowWindow(hwndMain, SW_HIDE);
    UpdateWindow(hwndMain);
}
