#include <iostream>
#include <strstream>
#include <fstream>
#include <stdio.h>
#include "view.h"
#include "glaunch.h"
#include "logfile.h"

using namespace GameLauncher;
using LogFile::gl_log;

Menu_View::Menu_View()
{
}

Menu_View::~Menu_View()
{
    delete p_quit_key;
    delete p_next_menu_key;
    delete p_previous_menu_key;
    delete p_null_listener;
    delete p_list;
    delete p_title;
    delete p_sub_title;
    delete p_container;
}

void
Menu_View::initialize(Engine * engine, FONT * font, int x, int y, int width,
                      int height)
{
    p_engine    = engine;
    p_font      = font;
    p_x         = x;
    p_y         = y;
    p_width     = width;
    p_height    = height;
    p_item_height       = text_height(font) + Text_VC::OUTLINE_PADDING;

    p_null_listener     = new Null_Menu_Listener();
    p_listener  = p_null_listener;
    p_container = new GEContainer();

    p_quit_key          = get_one_time_key(0);
    p_next_menu_key     = get_one_time_key(0);
    p_previous_menu_key = get_one_time_key(0);

    create_view_components();
    hide_split(0);
    p_list->set_listener(this);

    p_container->add_drawable(p_title, DRAW_PRIORITY);
    p_container->add_drawable(p_sub_title, DRAW_PRIORITY);
    p_container->add_drawable(p_separator, DRAW_PRIORITY);
    p_container->add_drawable(p_list, DRAW_PRIORITY);

    p_container->add_updatable(p_title, UPDATE_PRIORITY);
    p_container->add_updatable(p_sub_title, UPDATE_PRIORITY);
    p_container->add_updatable(p_list, UPDATE_PRIORITY);
}

Keypress_Watcher *
Menu_View::get_one_time_key(int scancode)
{
    Keypress_Watcher * watcher;
    watcher =
        p_engine->get_keypress_factory()->create_keypress_watcher(scancode);
    watcher->set_one_time();
    return watcher;
}

void
Menu_View::set_titles(const string & title, const string & sub_title)
{
    p_title->set_contents(title);
    p_sub_title->set_contents(sub_title);
}

void
Menu_View::set_select_joy(int joystick_event)
{
      p_list->set_select_joy(joystick_event);
}

void
Menu_View::set_line_down_joy(int joystick_event)
{
    p_list->set_line_down_joy(joystick_event);
}

void
Menu_View::set_line_up_joy(int joystick_event)
{
    p_list->set_line_up_joy(joystick_event);
}

void
Menu_View::set_page_down_joy(int joystick_event)
{
    p_list->set_page_down_joy(joystick_event);
}

void
Menu_View::set_page_up_joy(int joystick_event)
{
    p_list->set_page_up_joy(joystick_event);
}

void
Menu_View::set_line_up_key(int scancode)
{
    p_list->set_line_up_key(scancode);
}

void
Menu_View::set_line_down_key(int scancode)
{
    p_list->set_line_down_key(scancode);
}

void
Menu_View::set_page_up_key(int scancode)
{
    p_list->set_page_up_key(scancode);
}

void
Menu_View::set_page_down_key(int scancode)
{
    p_list->set_page_down_key(scancode);
}

void
Menu_View::set_select_key(int scancode)
{
    p_list->set_select_key(scancode);
}

void
Menu_View::set_quit_key(int scancode)
{
    p_quit_key->set_scancode(scancode);
}

void
Menu_View::set_next_menu_key(int scancode)
{
    p_next_menu_key->set_scancode(scancode);
}

void
Menu_View::set_previous_menu_key(int scancode)
{
    p_previous_menu_key->set_scancode(scancode);
}

void
Menu_View::set_key_repeat_delay(int initial, int repeat)
{
    p_list->set_key_repeat_delay(initial, repeat);
}

void
Menu_View::set_joystick_repeat_delay(int initial, int repeat)
{
    p_list->set_joystick_repeat_delay(initial, repeat);
}

void
Menu_View::set_use_animations(bool use_animations)
{
    p_list->set_use_animations(use_animations);
}

void
Menu_View::set_turbo_scroll_threshold(int threshold)
{
    p_list->set_turbo_scroll_threshold(threshold);
}

void
Menu_View::set_mouse_input_axis(Mouse_Input_Axis axis)
{
    p_list->set_mouse_input_axis(axis);
}

void
Menu_View::set_mouse_input_threshold(int threshold)
{
    p_list->set_mouse_input_threshold(threshold);
}

void
Menu_View::set_listener(Menu_Listener * listener)
{
    if (listener != 0)
        p_listener      = listener;
    else
        p_listener      = p_null_listener;
}

void
Menu_View::set_all_items(const Scrollable_Item_List & items,
                         unsigned selected_index, unsigned first_shown_index)
{
    p_list->set_all_items(items, selected_index, first_shown_index);
}

void
Menu_View::set_item_at(const Scrollable_Item & item, unsigned index)
{
    p_list->set_item_at(item, index);
}

unsigned
Menu_View::get_selected_index()
{
    return p_list->get_selected_index();
}

unsigned
Menu_View::get_first_shown_index()
{
    return p_list->get_first_shown_index();
}

void
Menu_View::set_selected_colors(int body, int outline)
{
    p_list->set_selected_colors(body, outline);
}

void
Menu_View::set_unselected_colors(int body, int outline)
{
    p_list->set_unselected_colors(body, outline);
    p_title->set_colors(body, outline);
    p_sub_title->set_colors(body, outline);
    p_separator->set_color(body);
}


void
Menu_View::draw(BITMAP * frame)
{
    p_container->draw(frame);
}

bool
Menu_View::update()
{
    bool        container_updated;

    container_updated   = p_container->update();

    if (p_state == HIDDEN)
    {
        p_listener->stopped_moving();
    }

    if (p_state == GOING_INTO_HIDING)
    {
        if (p_list->are_all_items_stopped())
        {
            p_state = HIDDEN;
        }
    }

    if (p_state == COMING_OUT_OF_HIDING)
    {
        if (p_list->are_all_items_stopped())
        {
            p_state = UNHIDDEN;
            p_listener->stopped_moving();
        }
    }

    if (p_state == SCROLLING)
    {
        if (p_list->are_all_items_stopped())
        {
            p_state = UNHIDDEN;
            p_listener->stopped_moving();
        }
    }

    if (p_state == UNHIDDEN)
        handle_keypresses();

    return container_updated;
}

void
Menu_View::handle_keypresses()
{
    if (p_quit_key->is_pressed())
        p_listener->quit_pressed();

    if (p_next_menu_key->is_pressed())
        p_listener->next_menu_pressed();

    if (p_previous_menu_key->is_pressed())
        p_listener->previous_menu_pressed();
}

void
Menu_View::hide_titles(int milliseconds)
{
    p_title->move_to(p_title_hidden_x, p_title_hidden_y, milliseconds);
    p_sub_title->move_to(p_sub_title_hidden_x, p_sub_title_hidden_y,
                         milliseconds);
}

void
Menu_View::unhide_titles(int milliseconds)
{
    p_title->move_to(p_title_x, p_title_y, milliseconds);
    p_sub_title->move_to(p_sub_title_x, p_sub_title_y, milliseconds);
}

void
Menu_View::selection_changed(unsigned index)
{
    p_listener->selection_changed(index);
    p_state = SCROLLING;
}

void
Menu_View::item_selected(unsigned index)
{
    p_listener->item_selected(index);
}

void
Menu_View::item_visible(unsigned index)
{
    p_listener->item_visible(index);
}

void
Menu_View::hide_left(int milliseconds)
{
    hide_titles(milliseconds);
    p_list->hide_left(milliseconds);
    p_state = GOING_INTO_HIDING;
}

void
Menu_View::hide_right(int milliseconds)
{
    hide_titles(milliseconds);
    p_list->hide_right(milliseconds);
    p_state = GOING_INTO_HIDING;
}

void
Menu_View::hide_split(int milliseconds)
{
    hide_titles(milliseconds);
    p_list->hide_split(milliseconds);
    p_state = GOING_INTO_HIDING;
}

void
Menu_View::center_selected_hide_others(int milliseconds)
{
    hide_titles(milliseconds);
    p_list->center_selected_hide_others(milliseconds);
    p_state = GOING_INTO_HIDING;
}

void
Menu_View::unhide_all(int milliseconds)
{
    unhide_titles(milliseconds);
    p_list->unhide_all(milliseconds);
    p_state = COMING_OUT_OF_HIDING;
}


void
Null_Menu_Listener::selection_changed(unsigned index)
{
}

void
Null_Menu_Listener::item_selected(unsigned index)
{
}

void
Null_Menu_Listener::item_visible(unsigned index)
{
}

void
Null_Menu_Listener::quit_pressed()
{
}

void
Null_Menu_Listener::next_menu_pressed()
{
}

void
Null_Menu_Listener::previous_menu_pressed()
{
}

void
Null_Menu_Listener::stopped_moving()
{
}


HMenu_View::HMenu_View(Engine * engine, FONT * font, int x, int y, int width,
                       int height)
    : Menu_View()
{
    initialize(engine, font, x, y, width, height);
}

void
HMenu_View::create_view_components()
{
    int list_x;
    int list_y;
    int list_width;
    int list_height;

    p_title_x           = p_x;
    p_title_y           = p_y;

    p_title_hidden_x    = p_x;
    p_title_hidden_y    = 0 - 2*p_item_height;

    p_sub_title_x       = p_x;
    p_sub_title_y       = p_title_y + p_item_height;

    p_sub_title_hidden_x        = p_x;
    p_sub_title_hidden_y        = 0 - p_item_height;

    p_separator_x       = p_x;
    p_separator_y       = p_sub_title_y + p_item_height;

    list_x              = p_x;
    list_y              = p_separator_y + SEPARATOR_HEIGHT;

    list_width          = p_width;
    list_height         = p_height - 2*p_item_height - SEPARATOR_HEIGHT;

    p_title     = new HText_VC(p_engine, p_width, p_item_height, p_font);
    p_sub_title = new HText_VC(p_engine, p_width, p_item_height, p_font);
    p_separator = new Outline_Bar(p_separator_x, p_separator_y,
                                  p_separator_x + p_width,
                                  p_separator_y + SEPARATOR_HEIGHT);
    p_list      = new Scrollable_HList_VC(p_engine, p_font,
                                          list_x, list_y,
                                          list_width, list_height);
}



VLMenu_View::VLMenu_View(Engine * engine, FONT * font, int x, int y, int width,
                         int height)
    : Menu_View()
{
    initialize(engine, font, x, y, width, height);
}

void
VLMenu_View::create_view_components()
{
    int list_x;
    int list_y;
    int list_width;
    int list_height;

    p_title_x           = p_x;
    p_title_y           = p_y;

    p_title_hidden_x    = 0 - 2*p_item_height;
    p_title_hidden_y    = p_y;

    p_sub_title_x       = p_title_x + p_item_height;
    p_sub_title_y       = p_y;

    p_sub_title_hidden_x        = 0 - p_item_height;
    p_sub_title_hidden_y        = p_y;

    p_separator_x       = p_sub_title_x + p_item_height;
    p_separator_y       = p_y;

    list_x              = p_separator_x + SEPARATOR_HEIGHT;
    list_y              = p_y;

    list_width          = p_width - 2*p_item_height - SEPARATOR_HEIGHT;
    list_height         = p_height;

    p_title     = new VText_VC(p_engine, p_item_height, p_height, p_font);
    p_sub_title = new VText_VC(p_engine, p_item_height, p_height, p_font);
    p_separator = new Outline_Bar(p_separator_x, p_separator_y,
                                  p_separator_x + SEPARATOR_HEIGHT,
                                  p_separator_y + p_height);
    p_list      = new Scrollable_VLList_VC(p_engine, p_font,
                                           list_x, list_y,
                                           list_width, list_height);
}




#ifdef MENU_TEST
Test_TVC::Test_TVC(Engine * engine, FONT * font)
{
    GEContainer *       root;

    p_engine = engine;
    p_font = font;
    root = engine->get_root_container();
    p_tvc = new VText_VC(engine, text_height(font), 400, font);
    p_tvc->set_colors(makecol(255, 0, 0), makecol(0, 0, 255));
    p_tvc->set_contents("Hello!");
    root->add_drawable(p_tvc, 100);
    root->add_updatable(p_tvc, 100);
    p_icon = create_bitmap(20, 10);
    clear_to_color(p_icon, makecol(0, 255, 0));
}

bool
Test_TVC::update()
{
    GEContainer *       root;

    root = p_engine->get_root_container();
    if (key[KEY_D])
    {
        p_tvc->set_colors(makecol(255, 0, 0), makecol(0, 0, 255));
        p_tvc->set_contents("Hello!");
        p_tvc->move_to(0, 0);
    }
    else if (key[KEY_A])
    {
        p_tvc->set_colors(makecol(0, 0, 255), makecol(255, 0, 0));
        p_tvc->move_to(SCREEN_W - 100, SCREEN_H - 150, 500);
    }
    else if (key[KEY_B])
    {
        p_tvc->set_contents("Bye...", p_icon);
        p_tvc->move_to(SCREEN_W - 100, 0, 100);
    }
    else if (key[KEY_C])
    {
        p_tvc->move_to(0, SCREEN_H - 150, 1000);
    }
    else if (key[KEY_X])
    {
        root->remove_drawable(p_tvc);
        root->remove_updatable(p_tvc);
        delete p_tvc;
        p_tvc = new HText_VC(p_engine, 400,
                             text_height(p_font)+Text_VC::OUTLINE_PADDING,
                             p_font);
        p_tvc->set_colors(makecol(255, 0, 0), makecol(0, 0, 255));
        p_tvc->set_contents("Hello!");
        p_tvc->set_clip(100, 100, SCREEN_W - 100, SCREEN_H - 100);
        root->add_drawable(p_tvc, 100);
        root->add_updatable(p_tvc, 100);
    }
    else if (key[KEY_Y])
    {
        root->remove_drawable(p_tvc);
        root->remove_updatable(p_tvc);
        delete p_tvc;
        p_tvc = new VText_VC(p_engine,
                             text_height(p_font)+Text_VC::OUTLINE_PADDING,
                             400, p_font);
        p_tvc->set_colors(makecol(255, 0, 0), makecol(0, 0, 255));
        p_tvc->set_contents("Hello!");
        p_tvc->set_clip(100, 100, SCREEN_W - 100, SCREEN_H - 100);
        root->add_drawable(p_tvc, 100);
        root->add_updatable(p_tvc, 100);
    }

    return true;
}


Test_SL::Test_SL(Engine * engine, FONT * font)
{
    p_engine    = engine;
    p_font      = font;

    p_items.resize(NUM_ITEMS);

    int i;
    for (i = 0; i < NUM_ITEMS; i++)
    {
        set_item(i);
    }

    p_menu = new HMenu_View(engine, font, 0, 100, SCREEN_W, SCREEN_H - 150);

    p_menu->set_selected_colors(makecol(255, 0, 0), makecol(0, 255, 0));
    p_menu->set_unselected_colors(makecol(0, 255, 255), makecol(0, 0, 255));
    p_menu->set_line_up_key(KEY_UP);
    p_menu->set_line_down_key(KEY_DOWN);
    p_menu->set_page_up_key(KEY_PGUP);
    p_menu->set_page_down_key(KEY_PGDN);
    p_menu->set_select_key(KEY_1);

    p_menu->set_all_items(p_items, 37, 30);
    p_menu->set_titles("This is a", "Test, test test!");

    GEContainer *       root;
    root = p_engine->get_root_container();

    root->add_drawable(p_menu, 300);
    root->add_updatable(p_menu, 300);
}

Test_SL::~Test_SL()
{
    delete p_menu;
}

void
Test_SL::set_item(unsigned item)
{
    // Dammit!  Why aren't there ostringstreams??  And why don't
    // ostrstreams initialize their damn buffer!!
    char            buf[81];
    memset(buf, 0, 81);
    ostrstream      text(buf, 80);

    text << "Item " << item;
    p_items[item].text     = text.str();
    p_items[item].icon     = 0;
    ks = p_engine->get_keypress_factory()->create_keypress_watcher(KEY_S);
    kd = p_engine->get_keypress_factory()->create_keypress_watcher(KEY_D);
    kf = p_engine->get_keypress_factory()->create_keypress_watcher(KEY_F);
    ke = p_engine->get_keypress_factory()->create_keypress_watcher(KEY_E);
    ks->set_one_time();
    kd->set_one_time();
    kf->set_one_time();
    ke->set_one_time();

}

bool
Test_SL::update()
{
    if (kd->is_pressed())
    {
        p_menu->unhide_all(1000);
    }
    if (kf->is_pressed())
    {
        p_menu->hide_right(1000);
    }
    if (ks->is_pressed())
    {
        p_menu->hide_left(1000);
    }
    if (ke->is_pressed())
    {
        p_menu->hide_split(1000);
    }

    return true;
}
#endif
