#ifndef _MP3BGI_H_
#define _MP3BGI_H_

#include <vector>
#include <string>
#include "almp3.h"
#include "mp3bg.h"

namespace GameLauncher
{
    using namespace GameEngine;

    class MP3_Background_Almp3_File : public MP3_Background
    {
      public:
        MP3_Background_Almp3_File() throw ();
        virtual ~MP3_Background_Almp3_File();
        
        virtual void        poll();
        virtual void        resume();
        virtual void        suspend();

        virtual void        set_playlist(const vector<string> & playlist);
        virtual void        set_volume(int i);
        virtual int         get_current_position() const;
        virtual string      get_current_mp3_name() const;
        virtual unsigned    get_current_mp3_index() const;
        
        virtual void        seek(unsigned index, int position = 0);
        virtual void skip_to_next_song();
        virtual void start_song_over();

      private:
        typedef vector<string>  Playlist_Impl;
        static const int        BUFFER_SIZE         = 16 * 1024;
        static const int        MAX_VOLUME          = 255;
        static const int        PAN                 = 127;
        static const int        SPEED               = 1000;
        static const bool       NO_LOOP             = false;

        bool            open_current_mp3();
        void            close_current_mp3();

        void            read_mp3_data(const string & filename);
        void            free_mp3_data();
        void            goto_next_mp3();

        Playlist_Impl   p_playlist;
        unsigned int    p_current_mp3_index;
        bool            p_is_suspended;
        
        ALMP3_MP3 *     p_current_mp3;
        unsigned char * p_mp3_data;
        int             p_current_position;
        int             p_volume;
    };

    class MP3_Background_Almp3_Stream : public MP3_Background
    {
      public:
        MP3_Background_Almp3_Stream() throw ();
        virtual ~MP3_Background_Almp3_Stream();
        
        virtual void        poll();
        virtual void        resume();
        virtual void        suspend();

        virtual void        set_playlist(const vector<string> & playlist);
        virtual void        set_volume(int i);
        virtual int         get_current_position() const;
        virtual string      get_current_mp3_name() const;
        virtual unsigned    get_current_mp3_index() const;
        
        virtual void        seek(unsigned index, int position = 0);
        virtual void skip_to_next_song();
        virtual void start_song_over();

      private:
        typedef vector<string>  Playlist_Impl;
        static const int        BLOCK_SIZE          = 32 * 1024;
        static const int        BUFFER_SIZE         = 24 * 1024;
        static const int        MAX_VOLUME          = 255;
        static const int        PAN                 = 127;
        static const int        SPEED               = 1000;

        bool            open_current_mp3();
        void            close_current_mp3();

        void            read_mp3_data(const string & filename);
        void            free_mp3_data();
        void            goto_next_mp3();

        Playlist_Impl   p_playlist;
        unsigned int    p_current_mp3_index;
        bool            p_is_suspended;
        
        ALMP3_MP3STREAM *   p_current_mp3;
        PACKFILE *          p_mp3_file;
        int                 p_initial_offset;
        int                 p_current_position;
        int                 p_volume;
    };
}

#endif

/* Local Variables: */
/* mode: c++ */
/* End: */
